package com.tincancamera.asm.beans;

import java.lang.reflect.*;
import java.util.*;
import org.objectweb.asm.*;
import static org.objectweb.asm.Opcodes.*;

public class ComparatorFactory extends ClassLoader {
    private final Map<String, Comparator<?>> comparators = new HashMap<String, Comparator<?>>();

    public ComparatorFactory () {
    }

    @SuppressWarnings("unchecked")
    public Comparator createComparator (Class<?> beanType, String propertyName) throws Exception {
        final String id = beanType.getName() + "@@@" + propertyName;
        Comparator comparator = comparators.get(id);

        if (comparator == null)  {
            final String upcasePropertyName = Character.toUpperCase(propertyName.charAt(0)) + propertyName.substring(1);
            final String comparatorName = beanType.getName() + upcasePropertyName + "Comparator";
            final byte[] bytecode = createBytecode(beanType, upcasePropertyName);
            final Class<?> clz = defineClass(comparatorName, bytecode, 0, bytecode.length);

            comparator = (Comparator)clz.newInstance();

            comparators.put(id, comparator);
        }

        return comparator;
    }

    public static byte[] createBytecode (Class<?> beanType, String upcasePropertyName) throws Exception {
        ClassWriter cw = new ClassWriter(0);

        final String beanTypeName   = beanType.getName().replace('.', '/');
        final String comparatorName = beanTypeName + upcasePropertyName + "Comparator";

        final String accessorName      = "get" + upcasePropertyName;

        final Class<?> valueType = beanType.getDeclaredMethod(accessorName).getReturnType();

        final String valueTypeName;

        if (valueType.isPrimitive()) {
            if (valueType == Boolean.TYPE) {
                valueTypeName = "J";
            } else if (valueType == Character.TYPE) {
                valueTypeName = "C";
            }

            throw new IllegalArgumentException("Only comparable object values please");
        } else {
            valueTypeName = valueType.getName().replace('.', '/');
        }

        final String accessorSignature = "()L" + valueTypeName + ";";

        cw.visit(V1_6, ACC_PUBLIC + ACC_SUPER,
            comparatorName,
            "Ljava/lang/Object;Ljava/util/Comparator<Ljava/lange/Object;>;",
            "java/lang/Object", new String[] { "java/util/Comparator" });

        // constructor
        {
            final MethodVisitor mv = cw.visitMethod(ACC_PUBLIC, "<init>", "()V", null, null);
            mv.visitCode();
            mv.visitVarInsn(ALOAD, 0);
            mv.visitMethodInsn(INVOKESPECIAL, "java/lang/Object", "<init>", "()V");
            mv.visitInsn(RETURN);
            mv.visitMaxs(1, 1);
            mv.visitEnd();
        }

        // compare (Object left, Object right)
        {
            final MethodVisitor mv = cw.visitMethod(ACC_PUBLIC, "compare", "(Ljava/lang/Object;Ljava/lang/Object;)I", null, null);

            mv.visitCode();
            mv.visitVarInsn(ALOAD, 1);
            mv.visitTypeInsn(CHECKCAST, beanTypeName);
            mv.visitMethodInsn(INVOKEVIRTUAL, beanTypeName, accessorName, accessorSignature);
            mv.visitVarInsn(ASTORE, 3);
            mv.visitVarInsn(ALOAD, 2);
            mv.visitTypeInsn(CHECKCAST, beanTypeName);
            mv.visitMethodInsn(INVOKEVIRTUAL, beanTypeName, accessorName, accessorSignature);
            mv.visitVarInsn(ASTORE, 4);
            mv.visitVarInsn(ALOAD, 3);
            Label l0 = new Label();
            mv.visitJumpInsn(IFNONNULL, l0);
            mv.visitVarInsn(ALOAD, 4);
            Label l1 = new Label();
            mv.visitJumpInsn(IFNONNULL, l1);
            mv.visitInsn(ICONST_0);
            mv.visitInsn(IRETURN);
            mv.visitLabel(l1);
            mv.visitFrame(Opcodes.F_APPEND,2, new Object[] {"java/lang/Comparable", "java/lang/Comparable"}, 0, null);
            mv.visitInsn(ICONST_1);
            mv.visitInsn(IRETURN);
            mv.visitLabel(l0);
            mv.visitFrame(Opcodes.F_SAME, 0, null, 0, null);
            mv.visitVarInsn(ALOAD, 4);
            Label l2 = new Label();
            mv.visitJumpInsn(IFNONNULL, l2);
            mv.visitInsn(ICONST_M1);
            mv.visitInsn(IRETURN);
            mv.visitLabel(l2);
            mv.visitFrame(Opcodes.F_SAME, 0, null, 0, null);
            mv.visitVarInsn(ALOAD, 3);
            mv.visitVarInsn(ALOAD, 4);
            mv.visitMethodInsn(INVOKEINTERFACE, "java/lang/Comparable", "compareTo", "(Ljava/lang/Object;)I");
            mv.visitInsn(IRETURN);
            mv.visitMaxs(2, 5);
            mv.visitEnd();
        }

        cw.visitEnd();

        return cw.toByteArray();
    }

}
/*
package asm.com.tincancamera.asm.beans;
import java.util.*;
import org.objectweb.asm.*;
import org.objectweb.asm.attrs.*;
public class BeanNameComparatorDump implements Opcodes {

public static byte[] dump () throws Exception {

ClassWriter cw = new ClassWriter(0);
FieldVisitor fv;
MethodVisitor mv;
AnnotationVisitor av0;

cw.visit(V1_6, ACC_PUBLIC + ACC_SUPER, "com/tincancamera/asm/beans/BeanNameComparator", null, "java/lang/Object", new String[] { "java/util/Comparator" });

{
mv = cw.visitMethod(ACC_PUBLIC, "<init>", "()V", null, null);
mv.visitCode();
mv.visitVarInsn(ALOAD, 0);
mv.visitMethodInsn(INVOKESPECIAL, "java/lang/Object", "<init>", "()V");
mv.visitInsn(RETURN);
mv.visitMaxs(1, 1);
mv.visitEnd();
}
{
mv = cw.visitMethod(ACC_PUBLIC, "compare", "(Ljava/lang/Object;Ljava/lang/Object;)I", null, null);
mv.visitCode();
mv.visitVarInsn(ALOAD, 1);
mv.visitTypeInsn(CHECKCAST, "com/tincancamera/asm/beans/Bean");
mv.visitMethodInsn(INVOKEVIRTUAL, "com/tincancamera/asm/beans/Bean", "getName", "()Ljava/lang/String;");
mv.visitVarInsn(ASTORE, 3);
mv.visitVarInsn(ALOAD, 2);
mv.visitTypeInsn(CHECKCAST, "com/tincancamera/asm/beans/Bean");
mv.visitMethodInsn(INVOKEVIRTUAL, "com/tincancamera/asm/beans/Bean", "getName", "()Ljava/lang/String;");
mv.visitVarInsn(ASTORE, 4);
mv.visitVarInsn(ALOAD, 3);
Label l0 = new Label();
mv.visitJumpInsn(IFNONNULL, l0);
mv.visitVarInsn(ALOAD, 4);
Label l1 = new Label();
mv.visitJumpInsn(IFNONNULL, l1);
mv.visitInsn(ICONST_0);
mv.visitInsn(IRETURN);
mv.visitLabel(l1);
mv.visitFrame(Opcodes.F_APPEND,2, new Object[] {"java/lang/Comparable", "java/lang/Comparable"}, 0, null);
mv.visitInsn(ICONST_1);
mv.visitInsn(IRETURN);
mv.visitLabel(l0);
mv.visitFrame(Opcodes.F_SAME, 0, null, 0, null);
mv.visitVarInsn(ALOAD, 4);
Label l2 = new Label();
mv.visitJumpInsn(IFNONNULL, l2);
mv.visitInsn(ICONST_M1);
mv.visitInsn(IRETURN);
mv.visitLabel(l2);
mv.visitFrame(Opcodes.F_SAME, 0, null, 0, null);
mv.visitVarInsn(ALOAD, 3);
mv.visitVarInsn(ALOAD, 4);
mv.visitMethodInsn(INVOKEINTERFACE, "java/lang/Comparable", "compareTo", "(Ljava/lang/Object;)I");
mv.visitInsn(IRETURN);
mv.visitMaxs(2, 5);
mv.visitEnd();
}
{
mv = cw.visitMethod(0, "getCharacter", "()C", null, null);
mv.visitCode();
mv.visitInsn(ICONST_0);
mv.visitInsn(IRETURN);
mv.visitMaxs(1, 1);
mv.visitEnd();
}
{
mv = cw.visitMethod(0, "getByte", "()B", null, null);
mv.visitCode();
mv.visitInsn(ICONST_0);
mv.visitInsn(IRETURN);
mv.visitMaxs(1, 1);
mv.visitEnd();
}
{
mv = cw.visitMethod(0, "getShort", "()S", null, null);
mv.visitCode();
mv.visitInsn(ICONST_0);
mv.visitInsn(IRETURN);
mv.visitMaxs(1, 1);
mv.visitEnd();
}
{
mv = cw.visitMethod(0, "getInteger", "()I", null, null);
mv.visitCode();
mv.visitInsn(ICONST_0);
mv.visitInsn(IRETURN);
mv.visitMaxs(1, 1);
mv.visitEnd();
}
{
mv = cw.visitMethod(0, "getLong", "()J", null, null);
mv.visitCode();
mv.visitInsn(LCONST_0);
mv.visitInsn(LRETURN);
mv.visitMaxs(2, 1);
mv.visitEnd();
}
{
mv = cw.visitMethod(0, "getFloat", "()F", null, null);
mv.visitCode();
mv.visitInsn(FCONST_0);
mv.visitInsn(FRETURN);
mv.visitMaxs(1, 1);
mv.visitEnd();
}
{
mv = cw.visitMethod(0, "getDouble", "()D", null, null);
mv.visitCode();
mv.visitInsn(DCONST_0);
mv.visitInsn(DRETURN);
mv.visitMaxs(2, 1);
mv.visitEnd();
}
{
mv = cw.visitMethod(0, "getBoolean", "()Z", null, null);
mv.visitCode();
mv.visitInsn(ICONST_0);
mv.visitInsn(IRETURN);
mv.visitMaxs(1, 1);
mv.visitEnd();
}
{
mv = cw.visitMethod(ACC_PUBLIC, "compareCharacter", "(Lcom/tincancamera/asm/beans/BeanNameComparator;Lcom/tincancamera/asm/beans/BeanNameComparator;)I", null, null);
mv.visitCode();
mv.visitVarInsn(ALOAD, 1);
mv.visitMethodInsn(INVOKEVIRTUAL, "com/tincancamera/asm/beans/BeanNameComparator", "getCharacter", "()C");
mv.visitVarInsn(ISTORE, 3);
mv.visitVarInsn(ALOAD, 2);
mv.visitMethodInsn(INVOKEVIRTUAL, "com/tincancamera/asm/beans/BeanNameComparator", "getCharacter", "()C");
mv.visitVarInsn(ISTORE, 4);
mv.visitVarInsn(ILOAD, 3);
mv.visitVarInsn(ILOAD, 4);
Label l0 = new Label();
mv.visitJumpInsn(IF_ICMPNE, l0);
mv.visitInsn(ICONST_0);
mv.visitInsn(IRETURN);
mv.visitLabel(l0);
mv.visitFrame(Opcodes.F_APPEND,2, new Object[] {Opcodes.INTEGER, Opcodes.INTEGER}, 0, null);
mv.visitVarInsn(ILOAD, 3);
mv.visitVarInsn(ILOAD, 4);
Label l1 = new Label();
mv.visitJumpInsn(IF_ICMPGE, l1);
mv.visitInsn(ICONST_M1);
Label l2 = new Label();
mv.visitJumpInsn(GOTO, l2);
mv.visitLabel(l1);
mv.visitFrame(Opcodes.F_SAME, 0, null, 0, null);
mv.visitInsn(ICONST_1);
mv.visitLabel(l2);
mv.visitFrame(Opcodes.F_SAME1, 0, null, 1, new Object[] {Opcodes.INTEGER});
mv.visitInsn(IRETURN);
mv.visitMaxs(2, 5);
mv.visitEnd();
}
{
mv = cw.visitMethod(ACC_PUBLIC, "compareByte", "(Lcom/tincancamera/asm/beans/BeanNameComparator;Lcom/tincancamera/asm/beans/BeanNameComparator;)I", null, null);
mv.visitCode();
mv.visitVarInsn(ALOAD, 1);
mv.visitMethodInsn(INVOKEVIRTUAL, "com/tincancamera/asm/beans/BeanNameComparator", "getByte", "()B");
mv.visitVarInsn(ISTORE, 3);
mv.visitVarInsn(ALOAD, 2);
mv.visitMethodInsn(INVOKEVIRTUAL, "com/tincancamera/asm/beans/BeanNameComparator", "getByte", "()B");
mv.visitVarInsn(ISTORE, 4);
mv.visitVarInsn(ILOAD, 3);
mv.visitVarInsn(ILOAD, 4);
Label l0 = new Label();
mv.visitJumpInsn(IF_ICMPNE, l0);
mv.visitInsn(ICONST_0);
mv.visitInsn(IRETURN);
mv.visitLabel(l0);
mv.visitFrame(Opcodes.F_APPEND,2, new Object[] {Opcodes.INTEGER, Opcodes.INTEGER}, 0, null);
mv.visitVarInsn(ILOAD, 3);
mv.visitVarInsn(ILOAD, 4);
Label l1 = new Label();
mv.visitJumpInsn(IF_ICMPGE, l1);
mv.visitInsn(ICONST_M1);
Label l2 = new Label();
mv.visitJumpInsn(GOTO, l2);
mv.visitLabel(l1);
mv.visitFrame(Opcodes.F_SAME, 0, null, 0, null);
mv.visitInsn(ICONST_1);
mv.visitLabel(l2);
mv.visitFrame(Opcodes.F_SAME1, 0, null, 1, new Object[] {Opcodes.INTEGER});
mv.visitInsn(IRETURN);
mv.visitMaxs(2, 5);
mv.visitEnd();
}
{
mv = cw.visitMethod(ACC_PUBLIC, "compareShort", "(Lcom/tincancamera/asm/beans/BeanNameComparator;Lcom/tincancamera/asm/beans/BeanNameComparator;)I", null, null);
mv.visitCode();
mv.visitVarInsn(ALOAD, 1);
mv.visitMethodInsn(INVOKEVIRTUAL, "com/tincancamera/asm/beans/BeanNameComparator", "getShort", "()S");
mv.visitVarInsn(ISTORE, 3);
mv.visitVarInsn(ALOAD, 2);
mv.visitMethodInsn(INVOKEVIRTUAL, "com/tincancamera/asm/beans/BeanNameComparator", "getShort", "()S");
mv.visitVarInsn(ISTORE, 4);
mv.visitVarInsn(ILOAD, 3);
mv.visitVarInsn(ILOAD, 4);
Label l0 = new Label();
mv.visitJumpInsn(IF_ICMPNE, l0);
mv.visitInsn(ICONST_0);
mv.visitInsn(IRETURN);
mv.visitLabel(l0);
mv.visitFrame(Opcodes.F_APPEND,2, new Object[] {Opcodes.INTEGER, Opcodes.INTEGER}, 0, null);
mv.visitVarInsn(ILOAD, 3);
mv.visitVarInsn(ILOAD, 4);
Label l1 = new Label();
mv.visitJumpInsn(IF_ICMPGE, l1);
mv.visitInsn(ICONST_M1);
Label l2 = new Label();
mv.visitJumpInsn(GOTO, l2);
mv.visitLabel(l1);
mv.visitFrame(Opcodes.F_SAME, 0, null, 0, null);
mv.visitInsn(ICONST_1);
mv.visitLabel(l2);
mv.visitFrame(Opcodes.F_SAME1, 0, null, 1, new Object[] {Opcodes.INTEGER});
mv.visitInsn(IRETURN);
mv.visitMaxs(2, 5);
mv.visitEnd();
}
{
mv = cw.visitMethod(ACC_PUBLIC, "compareInteger", "(Lcom/tincancamera/asm/beans/BeanNameComparator;Lcom/tincancamera/asm/beans/BeanNameComparator;)I", null, null);
mv.visitCode();
mv.visitVarInsn(ALOAD, 1);
mv.visitMethodInsn(INVOKEVIRTUAL, "com/tincancamera/asm/beans/BeanNameComparator", "getInteger", "()I");
mv.visitVarInsn(ISTORE, 3);
mv.visitVarInsn(ALOAD, 2);
mv.visitMethodInsn(INVOKEVIRTUAL, "com/tincancamera/asm/beans/BeanNameComparator", "getInteger", "()I");
mv.visitVarInsn(ISTORE, 4);
mv.visitVarInsn(ILOAD, 3);
mv.visitVarInsn(ILOAD, 4);
Label l0 = new Label();
mv.visitJumpInsn(IF_ICMPNE, l0);
mv.visitInsn(ICONST_0);
mv.visitInsn(IRETURN);
mv.visitLabel(l0);
mv.visitFrame(Opcodes.F_APPEND,2, new Object[] {Opcodes.INTEGER, Opcodes.INTEGER}, 0, null);
mv.visitVarInsn(ILOAD, 3);
mv.visitVarInsn(ILOAD, 4);
Label l1 = new Label();
mv.visitJumpInsn(IF_ICMPGE, l1);
mv.visitInsn(ICONST_M1);
Label l2 = new Label();
mv.visitJumpInsn(GOTO, l2);
mv.visitLabel(l1);
mv.visitFrame(Opcodes.F_SAME, 0, null, 0, null);
mv.visitInsn(ICONST_1);
mv.visitLabel(l2);
mv.visitFrame(Opcodes.F_SAME1, 0, null, 1, new Object[] {Opcodes.INTEGER});
mv.visitInsn(IRETURN);
mv.visitMaxs(2, 5);
mv.visitEnd();
}
{
mv = cw.visitMethod(ACC_PUBLIC, "compareLong", "(Lcom/tincancamera/asm/beans/BeanNameComparator;Lcom/tincancamera/asm/beans/BeanNameComparator;)I", null, null);
mv.visitCode();
mv.visitVarInsn(ALOAD, 1);
mv.visitMethodInsn(INVOKEVIRTUAL, "com/tincancamera/asm/beans/BeanNameComparator", "getLong", "()J");
mv.visitVarInsn(LSTORE, 3);
mv.visitVarInsn(ALOAD, 2);
mv.visitMethodInsn(INVOKEVIRTUAL, "com/tincancamera/asm/beans/BeanNameComparator", "getLong", "()J");
mv.visitVarInsn(LSTORE, 5);
mv.visitVarInsn(LLOAD, 3);
mv.visitVarInsn(LLOAD, 5);
mv.visitInsn(LCMP);
Label l0 = new Label();
mv.visitJumpInsn(IFNE, l0);
mv.visitInsn(ICONST_0);
mv.visitInsn(IRETURN);
mv.visitLabel(l0);
mv.visitFrame(Opcodes.F_APPEND,2, new Object[] {Opcodes.LONG, Opcodes.LONG}, 0, null);
mv.visitVarInsn(LLOAD, 3);
mv.visitVarInsn(LLOAD, 5);
mv.visitInsn(LCMP);
Label l1 = new Label();
mv.visitJumpInsn(IFGE, l1);
mv.visitInsn(ICONST_M1);
Label l2 = new Label();
mv.visitJumpInsn(GOTO, l2);
mv.visitLabel(l1);
mv.visitFrame(Opcodes.F_SAME, 0, null, 0, null);
mv.visitInsn(ICONST_1);
mv.visitLabel(l2);
mv.visitFrame(Opcodes.F_SAME1, 0, null, 1, new Object[] {Opcodes.INTEGER});
mv.visitInsn(IRETURN);
mv.visitMaxs(4, 7);
mv.visitEnd();
}
{
mv = cw.visitMethod(ACC_PUBLIC, "compareFloat", "(Lcom/tincancamera/asm/beans/BeanNameComparator;Lcom/tincancamera/asm/beans/BeanNameComparator;)I", null, null);
mv.visitCode();
mv.visitVarInsn(ALOAD, 1);
mv.visitMethodInsn(INVOKEVIRTUAL, "com/tincancamera/asm/beans/BeanNameComparator", "getFloat", "()F");
mv.visitVarInsn(FSTORE, 3);
mv.visitVarInsn(ALOAD, 2);
mv.visitMethodInsn(INVOKEVIRTUAL, "com/tincancamera/asm/beans/BeanNameComparator", "getFloat", "()F");
mv.visitVarInsn(FSTORE, 4);
mv.visitVarInsn(FLOAD, 3);
mv.visitVarInsn(FLOAD, 4);
mv.visitInsn(FCMPL);
Label l0 = new Label();
mv.visitJumpInsn(IFNE, l0);
mv.visitInsn(ICONST_0);
mv.visitInsn(IRETURN);
mv.visitLabel(l0);
mv.visitFrame(Opcodes.F_APPEND,2, new Object[] {Opcodes.FLOAT, Opcodes.FLOAT}, 0, null);
mv.visitVarInsn(FLOAD, 3);
mv.visitVarInsn(FLOAD, 4);
mv.visitInsn(FCMPG);
Label l1 = new Label();
mv.visitJumpInsn(IFGE, l1);
mv.visitInsn(ICONST_M1);
Label l2 = new Label();
mv.visitJumpInsn(GOTO, l2);
mv.visitLabel(l1);
mv.visitFrame(Opcodes.F_SAME, 0, null, 0, null);
mv.visitInsn(ICONST_1);
mv.visitLabel(l2);
mv.visitFrame(Opcodes.F_SAME1, 0, null, 1, new Object[] {Opcodes.INTEGER});
mv.visitInsn(IRETURN);
mv.visitMaxs(2, 5);
mv.visitEnd();
}
{
mv = cw.visitMethod(ACC_PUBLIC, "compareDouble", "(Lcom/tincancamera/asm/beans/BeanNameComparator;Lcom/tincancamera/asm/beans/BeanNameComparator;)I", null, null);
mv.visitCode();
mv.visitVarInsn(ALOAD, 1);
mv.visitMethodInsn(INVOKEVIRTUAL, "com/tincancamera/asm/beans/BeanNameComparator", "getDouble", "()D");
mv.visitVarInsn(DSTORE, 3);
mv.visitVarInsn(ALOAD, 2);
mv.visitMethodInsn(INVOKEVIRTUAL, "com/tincancamera/asm/beans/BeanNameComparator", "getDouble", "()D");
mv.visitVarInsn(DSTORE, 5);
mv.visitVarInsn(DLOAD, 3);
mv.visitVarInsn(DLOAD, 5);
mv.visitInsn(DCMPL);
Label l0 = new Label();
mv.visitJumpInsn(IFNE, l0);
mv.visitInsn(ICONST_0);
mv.visitInsn(IRETURN);
mv.visitLabel(l0);
mv.visitFrame(Opcodes.F_APPEND,2, new Object[] {Opcodes.DOUBLE, Opcodes.DOUBLE}, 0, null);
mv.visitVarInsn(DLOAD, 3);
mv.visitVarInsn(DLOAD, 5);
mv.visitInsn(DCMPG);
Label l1 = new Label();
mv.visitJumpInsn(IFGE, l1);
mv.visitInsn(ICONST_M1);
Label l2 = new Label();
mv.visitJumpInsn(GOTO, l2);
mv.visitLabel(l1);
mv.visitFrame(Opcodes.F_SAME, 0, null, 0, null);
mv.visitInsn(ICONST_1);
mv.visitLabel(l2);
mv.visitFrame(Opcodes.F_SAME1, 0, null, 1, new Object[] {Opcodes.INTEGER});
mv.visitInsn(IRETURN);
mv.visitMaxs(4, 7);
mv.visitEnd();
}
{
mv = cw.visitMethod(ACC_PUBLIC, "compareBoolean", "(Lcom/tincancamera/asm/beans/BeanNameComparator;Lcom/tincancamera/asm/beans/BeanNameComparator;)I", null, null);
mv.visitCode();
mv.visitVarInsn(ALOAD, 1);
mv.visitMethodInsn(INVOKEVIRTUAL, "com/tincancamera/asm/beans/BeanNameComparator", "getBoolean", "()Z");
mv.visitVarInsn(ISTORE, 3);
mv.visitVarInsn(ALOAD, 2);
mv.visitMethodInsn(INVOKEVIRTUAL, "com/tincancamera/asm/beans/BeanNameComparator", "getBoolean", "()Z");
mv.visitVarInsn(ISTORE, 4);
mv.visitVarInsn(ILOAD, 3);
mv.visitVarInsn(ILOAD, 4);
Label l0 = new Label();
mv.visitJumpInsn(IF_ICMPNE, l0);
mv.visitInsn(ICONST_0);
mv.visitInsn(IRETURN);
mv.visitLabel(l0);
mv.visitFrame(Opcodes.F_APPEND,2, new Object[] {Opcodes.INTEGER, Opcodes.INTEGER}, 0, null);
mv.visitVarInsn(ILOAD, 3);
Label l1 = new Label();
mv.visitJumpInsn(IFNE, l1);
mv.visitInsn(ICONST_M1);
Label l2 = new Label();
mv.visitJumpInsn(GOTO, l2);
mv.visitLabel(l1);
mv.visitFrame(Opcodes.F_SAME, 0, null, 0, null);
mv.visitInsn(ICONST_1);
mv.visitLabel(l2);
mv.visitFrame(Opcodes.F_SAME1, 0, null, 1, new Object[] {Opcodes.INTEGER});
mv.visitInsn(IRETURN);
mv.visitMaxs(2, 5);
mv.visitEnd();
}
cw.visitEnd();

return cw.toByteArray();
}
}
*/
